import SocketService from "./socket";

const socketService = SocketService.getInstance();
class WebRtc {
    private mediaStream: MediaStream | Blob | null = null;
    private pee: RTCPeerConnection | null = null;
    private video: HTMLVideoElement | null = null;

    async init() {
        this.video = document.getElementById('rtcVideo') as HTMLVideoElement;
        socketService.on("message", (e) => {
            let msg = JSON.parse(e)
            switch (msg.type) {
                case "answer":
                    let answer = JSON.parse(msg.content.body)
                    if (!answer) {
                        return console.log('failed to parse answer')
                    }
                    console.log(answer)
                    this.setOffer(answer.data.description)
                    break;
                case "offer":
                    let offer = msg.data.description
                    this.pee?.setRemoteDescription(offer)
                    this.pee?.createAnswer().then(answer => {
                        this.pee?.setLocalDescription(answer)
                        socketService.send(JSON.stringify({
                            type: 'answer', data: {
                                'to': msg.data.from,
                                'from': "31283192",
                                'description': { 'sdp': answer.sdp, 'type': answer.type },
                                'session_id': msg.data.from + "-31283192",
                            }
                        }))
                    })
                    break;
                case 'candidate':
                    this.pee?.setRemoteDescription(msg.data.description)
                    break;
                default:
                    break;
            }
        });
        this.createOffer();
    }
    async createOffer() {
        try {
            const configuration = {
                iceServers: [{ urls: 'stun:127.0.0.1:19302' }]
            };
            const peerConnection = new RTCPeerConnection(configuration);
            this.pee = peerConnection
            // 获取远方流添加到页面播放
            peerConnection.ontrack = event => {
                const remoteVideo = document.querySelector('#remoteVideo') as HTMLVideoElement;
                remoteVideo.autoplay = true
                remoteVideo.controls = true
                remoteVideo.srcObject = event.streams[0];
                event.track.onmute = function (event) {
                    remoteVideo.play()
                }
            };
            await this.getMedia(peerConnection)
            this.sendOffer()
            peerConnection.onicecandidate = e => {
                if (!e.candidate) {
                    return
                }
                socketService.send({ type: 'candidate', data: e.candidate })
            }
        } catch (error) {
            console.log(error);
        }
    }
    sendOffer() {
        let that = this;
        that.pee?.createOffer().then(function (offer) {
            that.pee?.setLocalDescription(offer);
            socketService.send(JSON.stringify({
                type: "offer", "data": {
                    "to": "848401",
                    "from": "31283192",
                    "description": offer,
                    "media": "video",
                    "session_id": "848401-31283192",
                }
            }))
        }).catch(function (error) {
            // 错误处理
            console.error(error);
        });
    }
    setOffer(offer) {
        this.pee?.setRemoteDescription(offer)
    }
    addIceCandidate(candidate) {
        console.log('addIceCandidate');
        this.pee?.addIceCandidate(candidate)
    }
    async getMedia(pee: RTCPeerConnection) {
        try {
            const stream = await navigator.mediaDevices.getUserMedia({
                video: true,
                audio: true
            });
            this.mediaStream = stream;
            stream.getTracks().forEach(track => pee.addTrack(track, stream))
            this.calls();
        } catch (error) {
            console.log(error);
        }
    }
    calls() {
        if (this.video) {
            this.video.srcObject = this.mediaStream;
            this.video.autoplay = true;
        }
    }


    close() {
        this.video?.pause();
        (this.mediaStream as MediaStream)?.getTracks().forEach(track => track.stop());
    }
}

export const webRTC = new WebRtc()